SmartyCat = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

local maxMana, currMana, currInt, fsr, ngName, ngID, bearID

function SmartyCat:OnEnable()
	local _, class = UnitClass("player")
	if class ~= "DRUID" then
		return
	end
	
	CreateFrame("GameTooltip", "SC_Tooltip", UIParent, "GameTooltipTemplate")
	SC_Tooltip:SetOwner(UIParent, "ANCHOR_NONE")
	self:LEARNED_SPELL_IN_TAB()
	
	if UnitPowerType("player") == 0 then
		maxMana = UnitManaMax("player")
		currMana = UnitMana("player")
		_, currInt  = UnitStat("player", 4)
		self:Events()
		self:TriggerEvent("SmartyCat_ManaUpdated")
	else
		self:RegisterEvent("UNIT_DISPLAYPOWER", "WaitForShapeshift")
	end
end

function SmartyCat:WaitForShapeshift(a1)
	if a1 ~= "player" then 
		return
	end
	
	self:UnregisterEvent("UNIT_DISPLAYPOWER")
	maxMana = UnitManaMax("player")
	currMana = UnitMana("player")
	_, currInt  = UnitStat("player", 4)
	self:Events()
	self:TriggerEvent("SmartyCat_ManaUpdated")
end

function SmartyCat:Events()
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("UNIT_MAXMANA")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
end

function SmartyCat:PLAYER_ENTERING_WORLD()
	if UnitPowerType("player") ~= 0 then
		self:ScheduleEvent("SC_MaxMana", self.SetMax, 7, self)
	end
end

function SmartyCat:PLAYER_LEAVING_WORLD()
	self:CancelAllScheduledEvents()
end

--Update players max mana
function SmartyCat:UNIT_MAXMANA(a1)
	if a1 ~= "player" then 
		return
	end
	
	local _, int  = UnitStat("player", 4)
	if UnitPowerType("player") == 0 then
		maxMana = UnitManaMax("player")
		currMana = UnitMana("player")
		currInt = int
	elseif currInt ~= int then
		maxMana = maxMana + ((int - currInt) * 15)
		currInt = int
		if currMana > maxMana then
			currMana = maxMana
		end
	end
	
	self:TriggerEvent("SmartyCat_ManaUpdated")
end

local regenMana		--Dont count the first call to UNIT_MANA after shapeshifting

function SmartyCat:UNIT_MANA(a1)
	if a1 ~= "player" then
		return
	end
	
	if UnitPowerType("player") == 0 then
		self:CancelAllScheduledEvents()
		currMana = UnitMana("player")
		maxMana = UnitManaMax("player")
		self:TriggerEvent("SmartyCat_ManaUpdated")
	else
		if not self:IsEventScheduled("SC_Event") and regenMana then
			--Update mana every 2 seconds
			self:UpdateMana()
			self:ScheduleRepeatingEvent("SC_Event", self.UpdateMana, 2, self)
		end
		--if mana hasnt been updated for 7 seconds, set current mana to the max mana
		self:ScheduleEvent("SC_MaxMana", self.SetMax, 7, self)
	end
end

--Called when shapeshifting to Cat or Bear form
function SmartyCat:UNIT_DISPLAYPOWER(a1)
	if a1 ~= "player" then
		return
	end
	
	if UnitPowerType("player") == 0 then
		regenMana = false
		self:CancelAllScheduledEvents()
		currMana = UnitMana("player")
		maxMana = UnitManaMax("player")
	else
		regenMana = true
		--Five second rule
		fsr = true
		self:ScheduleEvent("FSR", function() fsr=false end, 5)
		--subtract the mana cost of shapeshifting
		SC_Tooltip:SetSpell(bearID, BOOKTYPE_SPELL)
		currMana = currMana - (SC_TooltipTextLeft2:GetText():match("(%d+)") or 0)
	end
	
	self:TriggerEvent("SmartyCat_ManaUpdated")
end

--Nature's Grasp
function SmartyCat:UNIT_SPELLCAST_SUCCEEDED(a1, a2, a3)
	if a1 == "player" and UnitPowerType("player") ~= 0 and a2 == ngName then
		fsr = true
		self:ScheduleEvent("FSR", function() fsr=false end, 5)
		for i = ngID, ngID + 7 do
			local _, rank = GetSpellName(i, BOOKTYPE_SPELL)
			if rank == a3 then
				SC_Tooltip:SetSpell(i, BOOKTYPE_SPELL)
				currMana = currMana - (SC_TooltipTextLeft2:GetText():match("(%d+)") or 0)
				self:TriggerEvent("SmartyCat_ManaUpdated")
				break
			end
		end
	end
end

--finds the spell ID's of Nature's Grasp and Bear Form
function SmartyCat:LEARNED_SPELL_IN_TAB()
    for i = 1, GetNumSpellTabs() do
        local _, texture, offset, numSpells = GetSpellTabInfo(i)
        if string.find(texture, "Spell_Nature_StarFall") then
        	for j = offset + 1, offset + numSpells do
        		if string.find(GetSpellTexture(j, BOOKTYPE_SPELL), "Spell_Nature_NaturesWrath") then
        			ngID = j
        			ngName = GetSpellName(j, BOOKTYPE_SPELL)
        			break
        		end
        	end
        elseif string.find(texture, "Ability_Racial_BearForm") then
        	for j = offset + 1, offset + numSpells do
        		if string.find(GetSpellTexture(j, BOOKTYPE_SPELL), "Ability_Racial_BearForm") then
        			bearID = j
        			break
        		end
        	end
        end
    end
end

function SmartyCat:UpdateMana()
	local regen, castingRegen = GetManaRegen()
	if fsr then
		--Five second rule mana regen
		currMana=currMana+floor(castingRegen*2+.5)
	else
		--Normal mana regen
		currMana=currMana+floor(regen*2+.5)
	end
	
	if currMana>=maxMana then
		currMana=maxMana
		self:CancelAllScheduledEvents()
	end
	
	self:TriggerEvent("SmartyCat_ManaUpdated")
end

function SmartyCat:SetMax()
	currMana=maxMana
	self:CancelAllScheduledEvents()
	self:TriggerEvent("SmartyCat_ManaUpdated")
end

function SmartyCat:GetManaPercent()
	if currMana then
		return floor((currMana / maxMana) * 100)
	else
		return 100
	end
end

function SmartyCat:GetManaValue()
	if currMana then
		return currMana / maxMana
	else
		return 1
	end
end

function SmartyCat:GetMaxMana()
	if maxMana then
		return maxMana
	else
		return 100
	end
end

function SmartyCat:GetCurrMana()
	if currMana then
		return currMana
	else
		return 100
	end
end